//
//  KisanVikasPatraViewController.swift
//  Product Calculator
//
//  Created by Suraj Kumar Mandal on 03/12/21.
//

import UIKit
import SideMenu
import Toast_Swift

class KisanVikasPatraViewController: UIViewController {
    
    @IBOutlet var depositDateTF: UITextField!
    @IBOutlet var depositAmountTF: UITextField!
    @IBOutlet var annualInterestRateTF: UITextField!
    @IBOutlet var termYearsTF: UITextField!
    @IBOutlet var termMonthsTF: UITextField!
    @IBOutlet var compoundFrequencyTF: UITextField!
    
    fileprivate let customPicker = ToolbarPickerView()
    var menu = Int()
    
    var viewModel = KisanVikasPatraViewModel()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Do any additional setup after loading the view.
        viewModel.delegate = self
        
        depositDateTF.delegate = self
        depositAmountTF.delegate = self
        annualInterestRateTF.delegate = self
        termYearsTF.delegate = self
        termMonthsTF.delegate = self
        compoundFrequencyTF.delegate = self
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        navigationController?.navigationBar.barStyle = .black
        setupUI()
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
    func setupUI() {
        self.navigationItem.title = AppConstant.MENU_TITLE[menu]
        self.navigationItem.hidesBackButton = true
        let menuBarButtonItem = UIBarButtonItem(image: UIImage(systemName: "line.3.horizontal"), style: .plain, target: self, action: #selector(openMenu))
        self.navigationItem.leftBarButtonItem  = menuBarButtonItem
        if #available(iOS 13, *) {
            let appearance = UINavigationBarAppearance()
            appearance.backgroundColor = .systemBlue
            appearance.titleTextAttributes = [.foregroundColor: UIColor.white]
            appearance.largeTitleTextAttributes = [.foregroundColor: UIColor.white]
            
            navigationController?.navigationBar.tintColor = .white
            navigationController?.navigationBar.standardAppearance = appearance
            navigationController?.navigationBar.compactAppearance = appearance
            navigationController?.navigationBar.scrollEdgeAppearance = appearance
        }
        
        //Date Picker
        depositDateTF.datePicker(target: self,
                                 doneAction: #selector(doneAction),
                                 cancelAction: #selector(cancelAction),
                                 datePickerMode: .date)
        depositDateTF.delegate = self
        
        //Picker View
        createPickerView()
    }
    
    @objc func openMenu(_ sender: Any) {
        let menu = storyboard!.instantiateViewController(withIdentifier: "SideMenuNavigationController") as! SideMenuNavigationController
        present(menu, animated: true, completion: nil)
    }
    
    func createPickerView() {
        self.compoundFrequencyTF.inputView = self.customPicker
        self.compoundFrequencyTF.inputAccessoryView = self.customPicker.toolbar
        
        self.customPicker.delegate = self
        self.customPicker.dataSource = self
        self.customPicker.toolbarDelegate = self
    }
    
    @objc
    func cancelAction() {
        self.depositDateTF.resignFirstResponder()
    }
    
    @objc
    func doneAction() {
        if let datePickerView = self.depositDateTF.inputView as? UIDatePicker {
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = AppConstant.dateFormat2
            let dateString = dateFormatter.string(from: datePickerView.date)
            self.depositDateTF.text = dateString
            
            print(datePickerView.date)
            print(dateString)
            
            self.depositDateTF.resignFirstResponder()
            if Reachability.isConnectedToNetwork() {
                viewModel.getKVPInterest(depositDate: dateString)
            } else {
                Alert.showInternetFailureAlert(on: self)
            }
        }
    }
    
    func dataValidation() {
        if depositDateTF.text?.isEmpty == true {
            self.view.makeToast("Select deposit date!")
        } else if depositAmountTF.text?.isEmpty == true {
            self.view.makeToast("Fill deposit amount!")
        } else if annualInterestRateTF.text?.isEmpty == true {
            self.view.makeToast("Fill annual interest rate!")
        } else if termYearsTF.text?.isEmpty == true {
            self.view.makeToast("Fill term years!")
        } else if termMonthsTF.text?.isEmpty == true {
            self.view.makeToast("Fill term months!")
        } else if compoundFrequencyTF.text?.isEmpty == true {
            self.view.makeToast("Select compound frequency!")
        } else {
            if Reachability.isConnectedToNetwork() {
                viewModel.getKVPOutput(depositAmount: depositAmountTF.text!, depositDate: depositDateTF.text!)
            } else {
                Alert.showInternetFailureAlert(on: self)
            }
        }
    }
    
    
    /*
     // MARK: - Navigation
     
     // In a storyboard-based application, you will often want to do a little preparation before navigation
     override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
     // Get the new view controller using segue.destination.
     // Pass the selected object to the new view controller.
     }
     */
    
    @IBAction func saveAction(_ sender: Any) {
        self.dataValidation()
    }
    
    @IBAction func undoAction(_ sender: Any) {
        depositDateTF.text = nil
        depositAmountTF.text = nil
        annualInterestRateTF.text = nil
//        termTF.text = nil
//        compoundFrequencyTF.text = nil
    }
    
}

extension KisanVikasPatraViewController: UIPickerViewDelegate, UIPickerViewDataSource {
    // MARK: UIPickerView Delegation
    
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    func pickerView( _ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        return AppConstant.INTEREST_PAYOUT_FREQUENCY.count
    }
    
    func pickerView( _ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        return AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
    }
    
    func pickerView( _ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        compoundFrequencyTF.text = AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
    }
}

extension KisanVikasPatraViewController: UITextFieldDelegate {
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        switch textField {
        case compoundFrequencyTF:
            return false
        default:
            return true
        }
    }
    
    // when user select a textfield, this method will be called
    func textFieldDidBeginEditing(_ textField: UITextField) {
        switch textField {
        case compoundFrequencyTF:
            customPicker.reloadAllComponents()
        default:
            break
        }
    }
}

extension KisanVikasPatraViewController: ToolbarPickerViewDelegate {
    func didTapDone() {
        let row = self.customPicker.selectedRow(inComponent: 0)
        self.customPicker.selectRow(row, inComponent: 0, animated: false)
        self.compoundFrequencyTF.text = AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
        self.compoundFrequencyTF.resignFirstResponder()
    }
    
    func didTapCancel() {
        self.compoundFrequencyTF.text = nil
        self.compoundFrequencyTF.resignFirstResponder()
    }
}

extension KisanVikasPatraViewController: KisanVikasPatraViewProtocol {
    func navigate(_ data: KisanVikasPatraModel) {
        DispatchQueue.main.async {
            let kvpSummaryVC = self.storyboard?.instantiateViewController(withIdentifier: "KisanVikasPatraSummaryViewController") as! KisanVikasPatraSummaryViewController
            kvpSummaryVC.kvpModel = data
            self.navigationController?.pushViewController(kvpSummaryVC, animated: true)
        }
    }
    
    func setInterestRate(rate: String) {
        annualInterestRateTF.text = rate
        termYearsTF.text = "9"
        termMonthsTF.text = "5"
        compoundFrequencyTF.text = "Annually"
    }
    
    func startLoader() {
        ActivityIndicator.start()
    }
    
    func stopLoader() {
        ActivityIndicator.stop()
    }
    
    func showError(error: String) {
        self.view.makeToast(error)
    }
}
