//
//  POTimeDepositViewController.swift
//  Product Calculator
//
//  Created by Suraj Kumar Mandal on 04/12/21.
//

import UIKit
import SideMenu
import Toast_Swift

class POTimeDepositViewController: UIViewController {
    
    @IBOutlet var depositAmountTF: UITextField!
    @IBOutlet var annualInterestRateTF: UITextField!
    @IBOutlet var termYearsTF: UITextField!
    @IBOutlet var interestPayoutFrequencyTF: UITextField!
    @IBOutlet var compoundFrequencyTF: UITextField!
    @IBOutlet var depositDateTF: UITextField!
    
    fileprivate let customPicker = ToolbarPickerView()
    var activeTextField = 0
    var menu = Int()
    
    var viewModel = POTimeDepositViewModel()

    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Do any additional setup after loading the view.
        viewModel.delegate = self
        
        depositAmountTF.delegate = self
        annualInterestRateTF.delegate = self
        termYearsTF.delegate = self
        interestPayoutFrequencyTF.delegate = self
        compoundFrequencyTF.delegate = self
        depositDateTF.delegate = self
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        navigationController?.navigationBar.barStyle = .black
        setupUI()
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
    func setupUI() {
        self.navigationItem.title = AppConstant.MENU_TITLE[menu]
        self.navigationItem.hidesBackButton = true
        let menuBarButtonItem = UIBarButtonItem(image: UIImage(systemName: "line.3.horizontal"), style: .plain, target: self, action: #selector(openMenu))
        self.navigationItem.leftBarButtonItem  = menuBarButtonItem
        if #available(iOS 13, *) {
            let appearance = UINavigationBarAppearance()
            appearance.backgroundColor = .systemBlue
            appearance.titleTextAttributes = [.foregroundColor: UIColor.white]
            appearance.largeTitleTextAttributes = [.foregroundColor: UIColor.white]
            
            navigationController?.navigationBar.tintColor = .white
            navigationController?.navigationBar.standardAppearance = appearance
            navigationController?.navigationBar.compactAppearance = appearance
            navigationController?.navigationBar.scrollEdgeAppearance = appearance
        }
        
        //Date Picker
        depositDateTF.datePicker(target: self,
                         doneAction: #selector(doneAction),
                         cancelAction: #selector(cancelAction),
                         datePickerMode: .date)
        depositDateTF.delegate = self
        
        //Picker View
        createPickerView()
    }
    
    @objc func openMenu(_ sender: Any) {
        let menu = storyboard!.instantiateViewController(withIdentifier: "SideMenuNavigationController") as! SideMenuNavigationController
        present(menu, animated: true, completion: nil)
    }
    
    func createPickerView() {
        self.interestPayoutFrequencyTF.inputView = self.customPicker
        self.interestPayoutFrequencyTF.inputAccessoryView = self.customPicker.toolbar
        
        self.compoundFrequencyTF.inputView = self.customPicker
        self.compoundFrequencyTF.inputAccessoryView = self.customPicker.toolbar
        
        self.customPicker.delegate = self
        self.customPicker.dataSource = self
        self.customPicker.toolbarDelegate = self
    }
    
    @objc
    func cancelAction() {
        self.depositDateTF.resignFirstResponder()
    }
    
    @objc
    func doneAction() {
        if let datePickerView = self.depositDateTF.inputView as? UIDatePicker {
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = AppConstant.dateFormat1
            let dateString = dateFormatter.string(from: datePickerView.date)
            self.depositDateTF.text = dateString
            //self.dateOfBirth = dateString
            
            print(datePickerView.date)
            print(dateString)
            
            self.depositDateTF.resignFirstResponder()
        }
    }
    
    func dataValidation() {
        if depositAmountTF.text?.isEmpty == true {
            self.view.makeToast("Fill deposit amount!")
        } else if annualInterestRateTF.text?.isEmpty == true {
            self.view.makeToast("Fill annual interest rate!")
        } else if termYearsTF.text?.isEmpty == true {
            self.view.makeToast("Select term in years!")
        } else if interestPayoutFrequencyTF.text?.isEmpty == true {
            self.view.makeToast("Select interest payout frequency!")
        } else if compoundFrequencyTF.text?.isEmpty == true {
            self.view.makeToast("Select compound frequency!")
        } else if depositDateTF.text?.isEmpty == true {
            self.view.makeToast("Select deposit date!")
        } else {
            if Reachability.isConnectedToNetwork() {
                let interest = annualInterestRateTF.text!
                let interestRate = Double(interest)!/100.0
                viewModel.getPoTimeOutput(deposit: depositDateTF.text!, yearsDays: termYearsTF.text!, compundingFreq: compoundFrequencyTF.text!, rdDepositFreq: interestPayoutFrequencyTF.text!, depositDate: depositDateTF.text!, interestRate: String(interestRate))
            } else {
                Alert.showInternetFailureAlert(on: self)
            }
        }
    }
    

    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destination.
        // Pass the selected object to the new view controller.
    }
    */
    
    @IBAction func saveAction(_ sender: Any) {
        self.dataValidation()
    }
    
    @IBAction func undoAction(_ sender: Any) {
        depositAmountTF.text = nil
        annualInterestRateTF.text = nil
        termYearsTF.text = nil
        interestPayoutFrequencyTF.text = nil
        compoundFrequencyTF.text = nil
        depositDateTF.text = nil
    }
    
}

extension POTimeDepositViewController: UIPickerViewDelegate, UIPickerViewDataSource {
    // MARK: UIPickerView Delegation
    
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    func pickerView( _ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        switch activeTextField
        {
        case 1:
            return AppConstant.INTEREST_PAYOUT_FREQUENCY.count
        case 2:
            return AppConstant.INTEREST_PAYOUT_FREQUENCY.count
        default:
            return 0
            
        }
    }
    
    func pickerView( _ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        switch activeTextField{
        case 1:
            return AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
        case 2:
            return AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
        default:
            return ""
        }
    }
    
    func pickerView( _ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        
        switch activeTextField{
        case 1:
            interestPayoutFrequencyTF.text = AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
            break
        case 2:
            compoundFrequencyTF.text = AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
            break
        default:
            break
        }
    }
}

extension POTimeDepositViewController: UITextFieldDelegate {
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        switch textField {
        case interestPayoutFrequencyTF:
            return false
        case compoundFrequencyTF:
            return false
        default:
            return true
        }
    }
    
    // when user select a textfield, this method will be called
    func textFieldDidBeginEditing(_ textField: UITextField) {
        switch textField {
        case interestPayoutFrequencyTF:
            activeTextField = 1
            customPicker.reloadAllComponents()
        case compoundFrequencyTF:
            activeTextField = 2
            customPicker.reloadAllComponents()
        default:
            activeTextField = 0
        }
    }
}

extension POTimeDepositViewController: ToolbarPickerViewDelegate {
    func didTapDone() {
        switch activeTextField {
        case 1:
            let row = self.customPicker.selectedRow(inComponent: 0)
            self.customPicker.selectRow(row, inComponent: 0, animated: false)
            self.interestPayoutFrequencyTF.text = AppConstant.DEPOSIT_TYPE[row]
            self.interestPayoutFrequencyTF.resignFirstResponder()
        case 2:
            let row = self.customPicker.selectedRow(inComponent: 0)
            self.customPicker.selectRow(row, inComponent: 0, animated: false)
            self.compoundFrequencyTF.text = AppConstant.FIXED_DEPOSIT_TERM[row]
            self.compoundFrequencyTF.resignFirstResponder()
        default:
            return
        }
    }
    
    func didTapCancel() {
        switch activeTextField {
        case 1:
            self.interestPayoutFrequencyTF.text = nil
            self.interestPayoutFrequencyTF.resignFirstResponder()
        case 2:
            self.compoundFrequencyTF.text = nil
            self.compoundFrequencyTF.resignFirstResponder()
        default:
            return
        }
    }
}

extension POTimeDepositViewController: POTimeDepositViewProtocol {
    func navigate(_ data:POTimeDepositModel) {
        DispatchQueue.main.async {
            let poTimeSummaryVC = self.storyboard?.instantiateViewController(withIdentifier: "POTimeSummaryViewController") as! POTimeSummaryViewController
            poTimeSummaryVC.poTimeModel = data
            self.navigationController?.pushViewController(poTimeSummaryVC, animated: true)
        }
    }
    
    func startLoader() {
        ActivityIndicator.start()
    }
    
    func stopLoader() {
        ActivityIndicator.stop()
    }
    
    func showError(error: String) {
        self.view.makeToast(error)
    }
}
