//
//  SukanyaSamriddhiViewController.swift
//  Product Calculator
//
//  Created by Suraj Kumar Mandal on 05/12/21.
//

import UIKit
import SideMenu
import Toast_Swift

class SukanyaSamriddhiViewController: UIViewController {
    
    @IBOutlet var depositDateTF: UITextField!
    @IBOutlet var girlChildAgeTF: UITextField!
    @IBOutlet var depositAmountTF: UITextField!
    @IBOutlet var amountDepositFrequencyTF: UITextField!
    @IBOutlet var annualInterestRateTF: UITextField!
    @IBOutlet var paymentTermTF: UITextField!
    @IBOutlet var maturityTermTF: UITextField!
    @IBOutlet var compoundFrequencyTF: UITextField!
    
    fileprivate let customPicker = ToolbarPickerView()
    var activeTextField = 0
    var menu = Int()
    
    var viewModel = SukanyaSamriddhiViewModel()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Do any additional setup after loading the view.
        viewModel.delegate = self
        
        depositDateTF.delegate = self
        girlChildAgeTF.delegate = self
        depositAmountTF.delegate = self
        amountDepositFrequencyTF.delegate = self
        annualInterestRateTF.delegate = self
        paymentTermTF.delegate = self
        maturityTermTF.delegate = self
        compoundFrequencyTF.delegate = self
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        navigationController?.navigationBar.barStyle = .black
        setupUI()
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
    func setupUI() {
        self.navigationItem.title = AppConstant.MENU_TITLE[menu]
        self.navigationItem.hidesBackButton = true
        let menuBarButtonItem = UIBarButtonItem(image: UIImage(systemName: "line.3.horizontal"), style: .plain, target: self, action: #selector(openMenu))
        self.navigationItem.leftBarButtonItem  = menuBarButtonItem
        if #available(iOS 13, *) {
            let appearance = UINavigationBarAppearance()
            appearance.backgroundColor = .systemBlue
            appearance.titleTextAttributes = [.foregroundColor: UIColor.white]
            appearance.largeTitleTextAttributes = [.foregroundColor: UIColor.white]
            
            navigationController?.navigationBar.tintColor = .white
            navigationController?.navigationBar.standardAppearance = appearance
            navigationController?.navigationBar.compactAppearance = appearance
            navigationController?.navigationBar.scrollEdgeAppearance = appearance
        }
        
        //Date Picker
        depositDateTF.datePicker(target: self,
                                 doneAction: #selector(doneAction),
                                 cancelAction: #selector(cancelAction),
                                 datePickerMode: .date)
        depositDateTF.delegate = self
        
        //Picker View
        createPickerView()
    }
    
    @objc func openMenu(_ sender: Any) {
        let menu = storyboard!.instantiateViewController(withIdentifier: "SideMenuNavigationController") as! SideMenuNavigationController
        present(menu, animated: true, completion: nil)
    }
    
    func createPickerView() {
        self.amountDepositFrequencyTF.inputView = self.customPicker
        self.amountDepositFrequencyTF.inputAccessoryView = self.customPicker.toolbar
        
        self.compoundFrequencyTF.inputView = self.customPicker
        self.compoundFrequencyTF.inputAccessoryView = self.customPicker.toolbar
        
        self.customPicker.delegate = self
        self.customPicker.dataSource = self
        self.customPicker.toolbarDelegate = self
    }
    
    @objc
    func cancelAction() {
        self.depositDateTF.resignFirstResponder()
    }
    
    @objc
    func doneAction() {
        if let datePickerView = self.depositDateTF.inputView as? UIDatePicker {
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = AppConstant.dateFormat2
            let dateString = dateFormatter.string(from: datePickerView.date)
            self.depositDateTF.text = dateString
            
            print(datePickerView.date)
            print(dateString)
            
            self.depositDateTF.resignFirstResponder()
            if Reachability.isConnectedToNetwork() {
                viewModel.getInterestRate(depositDate: dateString)
            } else {
                Alert.showInternetFailureAlert(on: self)
            }
        }
    }
    
    func dataValidation() {
        if depositDateTF.text?.isEmpty == true {
            self.view.makeToast("Select deposit date!")
        } else if girlChildAgeTF.text?.isEmpty == true {
            self.view.makeToast("Select girl child age!")
        } else if depositAmountTF.text?.isEmpty == true {
            self.view.makeToast("Fill deposit amount!")
        } else if amountDepositFrequencyTF.text?.isEmpty == true {
            self.view.makeToast("Select amount deposit frequency!")
        } else if annualInterestRateTF.text?.isEmpty == true {
            self.view.makeToast("Fill annual interest rate!")
        } else if paymentTermTF.text?.isEmpty == true {
            self.view.makeToast("Select payment term in years!")
        } else if maturityTermTF.text?.isEmpty == true {
            self.view.makeToast("Select maturity term in years!")
        } else if compoundFrequencyTF.text?.isEmpty == true {
            self.view.makeToast("Select compound frequency!")
        } else {
            if Reachability.isConnectedToNetwork() {
                viewModel.getSukanyaSamriddhiOutput(depositAmount: depositAmountTF.text!, tenureType: "Y", paymenttenure: paymentTermTF.text!, rdDepositFreq: amountDepositFrequencyTF.text!, compundingFreq: compoundFrequencyTF.text!, depositDate: depositDateTF.text!, maturitytenure: maturityTermTF.text!)
            } else {
                Alert.showInternetFailureAlert(on: self)
            }
        }
    }
    
    
    /*
     // MARK: - Navigation
     
     // In a storyboard-based application, you will often want to do a little preparation before navigation
     override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
     // Get the new view controller using segue.destination.
     // Pass the selected object to the new view controller.
     }
     */
    
    @IBAction func saveAction(_ sender: Any) {
        self.dataValidation()
    }
    
    @IBAction func undoAction(_ sender: Any) {
        depositDateTF.text = nil
        girlChildAgeTF.text = nil
        depositAmountTF.text = nil
        amountDepositFrequencyTF.text = nil
        annualInterestRateTF.text = nil
        paymentTermTF.text = nil
        maturityTermTF.text = nil
        compoundFrequencyTF.text = nil
    }
    
}

extension SukanyaSamriddhiViewController: UIPickerViewDelegate, UIPickerViewDataSource {
    // MARK: UIPickerView Delegation
    
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    func pickerView( _ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        switch activeTextField
        {
        case 1:
            return AppConstant.INTEREST_PAYOUT_FREQUENCY.count
        case 2:
            return AppConstant.INTEREST_PAYOUT_FREQUENCY.count
        default:
            return 0
            
        }
    }
    
    func pickerView( _ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        switch activeTextField{
        case 1:
            return AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
        case 2:
            return AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
        default:
            return ""
        }
    }
    
    func pickerView( _ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        
        switch activeTextField{
        case 1:
            amountDepositFrequencyTF.text = AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
            break
        case 2:
            compoundFrequencyTF.text = AppConstant.INTEREST_PAYOUT_FREQUENCY[row]
            break
        default:
            break
        }
    }
}

extension SukanyaSamriddhiViewController: UITextFieldDelegate {
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        switch textField {
        case amountDepositFrequencyTF:
            return false
        case compoundFrequencyTF:
            return false
        default:
            return true
        }
    }
    
    // when user select a textfield, this method will be called
    func textFieldDidBeginEditing(_ textField: UITextField) {
        switch textField {
        case amountDepositFrequencyTF:
            activeTextField = 1
            customPicker.reloadAllComponents()
        case compoundFrequencyTF:
            activeTextField = 2
            customPicker.reloadAllComponents()
        default:
            activeTextField = 0
        }
    }
}

extension SukanyaSamriddhiViewController: ToolbarPickerViewDelegate {
    func didTapDone() {
        switch activeTextField {
        case 1:
            let row = self.customPicker.selectedRow(inComponent: 0)
            self.customPicker.selectRow(row, inComponent: 0, animated: false)
            self.amountDepositFrequencyTF.text = AppConstant.DEPOSIT_TYPE[row]
            self.amountDepositFrequencyTF.resignFirstResponder()
        case 2:
            let row = self.customPicker.selectedRow(inComponent: 0)
            self.customPicker.selectRow(row, inComponent: 0, animated: false)
            self.compoundFrequencyTF.text = AppConstant.FIXED_DEPOSIT_TERM[row]
            self.compoundFrequencyTF.resignFirstResponder()
        default:
            return
        }
    }
    
    func didTapCancel() {
        switch activeTextField {
        case 1:
            self.amountDepositFrequencyTF.text = nil
            self.amountDepositFrequencyTF.resignFirstResponder()
        case 2:
            self.compoundFrequencyTF.text = nil
            self.compoundFrequencyTF.resignFirstResponder()
        default:
            return
        }
    }
}

extension SukanyaSamriddhiViewController: SukanyaSamriddhiViewProtocol {
    func navigate(_ data: SukanyaSamriddhiModel) {
        DispatchQueue.main.async {
            let sukanyaSummaryVC = self.storyboard?.instantiateViewController(withIdentifier: "SukanyaSamriddhiSummaryViewController") as! SukanyaSamriddhiSummaryViewController
            sukanyaSummaryVC.sukanyaModel = data
            self.navigationController?.pushViewController(sukanyaSummaryVC, animated: true)
        }
    }
    
    func setInterestRate(rate: String) {
        annualInterestRateTF.text = rate
        paymentTermTF.text = "15"
        maturityTermTF.text = "21"
        compoundFrequencyTF.text = "Annually"
    }
    
    func startLoader() {
        ActivityIndicator.start()
    }
    
    func stopLoader() {
        ActivityIndicator.stop()
    }
    
    func showError(error: String) {
        self.view.makeToast(error)
    }
}
