//
//  PONationalSavingViewModel.swift
//  Product Calculator
//
//  Created by Suraj Kumar Mandal on 02/12/21.
//

import Foundation
import Alamofire

class PONationalSavingViewModel {
    
    var delegate: PONationalSavingViewProtocol?
    
    var poNSCModel = [PONationalSavingModel]()
    
    //POST PO National Saving Certificates Interest API
    func getPoNSCInterest(depositDate:String) {
        if let delegate = delegate {
            delegate.startLoader()
            let url = ApiUrl.BASE_URL + ApiUrl.GET_PO_NATIONAL_SAVING_INTEREST_RATE
           
            let parameters: [String: String] = [
                "depositDate": depositDate
            ]
            print(parameters)
            
            AF.request(url, method: .post, parameters: parameters, encoding: JSONEncoding.default).responseJSON { response in
                
                switch response.result {
                case .success(let value):
                    delegate.stopLoader()
                    print(value)
                    let data = value as! NSDictionary
                    print(data)
                    DispatchQueue.main.async {
                        let interestRate = data["interestRate"] as! Double
                        
                        delegate.setInterestRate(rate: String(interestRate))
                    }
                    
                case .failure(let error):
                    delegate.stopLoader()
                    print(error)
                    //delegate.showError(error: error)
                }
            }
        }
    }
    
    //POST PO National Saving Certificates API
    func getPoNSCOutput(openingAmount:String, annualInterest:String, tenure:String, payoutFrequency:String, depositDate:String) {
        if let delegate = delegate {
            delegate.startLoader()
            let url = ApiUrl.BASE_URL + ApiUrl.GET_PO_NATIONAL_SAVING
            
            var interest = String()
            switch payoutFrequency {
            case "Monthly":
                interest = "12"
            case "Bi-Monthly":
                interest = "6"
            case "Quarterly":
                interest = "4"
            case "Triannually":
                interest = "3"
            case "Half Yearly":
                interest = "2"
            case "Annually":
                interest = "1"
            default:
                interest = ""
            }
                        
            let parameters: [String: String] = [
                "openingAmount": openingAmount,
                "annualInterest": annualInterest,
                "tenure": tenure,
                "payoutFrequency": interest,
                "depositDate": depositDate
            ]
            print(parameters)
            
            AF.request(url, method: .post, parameters: parameters, encoding: JSONEncoding.default).responseJSON { response in
                
                switch response.result {
                case .success(let value):
                    delegate.stopLoader()
                    print(value)
                    let data = value as! NSDictionary
                    print(data)
                    DispatchQueue.main.async {
                        let depositAmount = data["depositAmount"] as! Double
                        let annualInterest = data["annualInterest"] as! Double
                        let term = data["term"] as! Int
                        let compundFreq = data["compundFreq"] as! Double
                        let depositDate = data["depositDate"] as? String
                        let maturityAmount = data["maturityAmount"] as! Double
                        let interestReceived = data["interestReceived"] as! Double
                        let totalInterestReceived = data["totalInterestReceived"] as! Double
                        let maturityDate = data["maturityDate"] as! String
                        
                        let poNScLookupList = data["poNScLookupList"] as! [NSDictionary]
                        
                        let poNScData = PONationalSavingModel(depositDate: depositDate ?? "", maturityDate: maturityDate, term: term, depositAmount: depositAmount, annualInterest: annualInterest, compundFreq: compundFreq, maturityAmount: maturityAmount, totalInterestReceived: totalInterestReceived, interestReceived: interestReceived)
                        
                        for list in poNScLookupList {
                            let serialNo = list.value(forKey: "serialNo") as! NSNumber
                            let referenceDate = list.value(forKey: "referenceDate") as! String
                            let referenceMonth = list.value(forKey: "referenceMonth") as! String
                            let financialYear = list.value(forKey: "financialYear") as! String
                            let openingBal = list.value(forKey: "openingBal") as! Double
                            let amountDeposited = list.value(forKey: "amountDeposited") as! Double
                            let interestAccrued = list.value(forKey: "interestAccrued") as! Double
                            let interestCredited = list.value(forKey: "interestCredited") as! Double
                            let totalInterestReceived = list.value(forKey: "totalInterestReceived") as! Double
                            let closingBalance = list.value(forKey: "closingBalance") as! Double
                            let daysToMaturity = list.value(forKey: "daysToMaturity") as! NSNumber
                                                        
                            let list = PONSCLookupList(serialNo: Int(truncating: serialNo), openingBal: openingBal, amountDeposited: amountDeposited, interestAccrued: interestAccrued, totalInterestReceived: totalInterestReceived, interestCredited: interestCredited, daysToMaturity: Int(truncating: daysToMaturity), referenceDate: referenceDate, referenceMonth: referenceMonth, financialYear: financialYear, closingBalance: closingBalance)
                            
                            poNScData.poNScLookupList.append(list)
                        }
                        self.poNSCModel.append(poNScData)
                        delegate.navigate(poNScData)
                    }
                    
                case .failure(let error):
                    delegate.stopLoader()
                    print(error)
                    //delegate.showError(error: error)
                }
            }
        }
    }
    
}

protocol PONationalSavingViewProtocol {
    func startLoader()
    func stopLoader()
    func showError(error:String)
    func setInterestRate(rate:String)
    func navigate(_ data:PONationalSavingModel)
}
